<?php
error_reporting(E_ALL);
ini_set('display_errors', TRUE);

function abort($body, $status = NULL) {
	if ($status) header("HTTP/1.1 $status");
	print "$body\n";
	exit;
}

function placeholder_include($name, $params = [])
{
	$path = stream_resolve_include_path("$name.php");
	if (!file_exists($path)) {
		return '<strong class="warn"><em>'.$name.'</em> ontbreekt</strong>';
	}

	ob_start();
	$Page = $GLOBALS['Page'] . $GLOBALS['Args'];
	$Args = '';
	$Place = $GLOBALS['Place'];
	foreach ($params as $param) {
		if ($set = strpos($param, '=')) {
			$Place[ substr($param, 0, $set) ] = substr($param, $set + 1);
		}
		elseif (!empty($param)) {
			$Args .= '/'.$param;
		}
	}
	try {
		include "$name.php";
		return ob_get_clean();
	}
	catch (Exception $e) {
		return sprintf('<strong class="warn">%s</strong>',
			"fout in <em>$name</em>: {$e->getMessage()}"
		);
	}
}

function getoutput($blocks = [])
{
	$doc = ob_get_clean();

	if (!empty($blocks['warn'])) {
		$warn = '<p class="warn">[[warn]]</p>';
		if ($offset = strpos($doc, '</h2>')) {
			$doc = substr_replace($doc, "\n\n".$warn, $offset + 5, 0);
		}
		else {
			$doc = $warn . "\n\n" . $doc;
		}
	}

	# keep either login or logout parts depending on user level
	global $User;
	$hideclass = empty($User) ? 'login' : 'logout';
	$doc = preg_replace('{\s*<(p|li|span) class="'.$hideclass.'">.*?</\1>}s', '', $doc);

	return preg_replace_callback(
		'{ (?<! <!--BLOCK: ) \[\[ ([^] ]+) ([^]]*) \]\] }x',
		function ($sub) use ($blocks) {
			list ($placeholder, $name, $params) = $sub;
			if (isset($blocks[$name])) {
				$html = $blocks[$name];
			}
			else {
				$html = placeholder_include($name, explode(' ', $params));
			}
			return sprintf('<!--BLOCK:%s-->%s<!--/-->',
				is_numeric($name) ? '' : $placeholder, # edit replacement
				preg_replace('{<!--[^-]*-->}', '', $html) # contents
			);
		},
		$doc
	);
}

# custom error handling

define('DOCROOT', getcwd());
set_include_path(implode(PATH_SEPARATOR, [ DOCROOT, __DIR__ ]));

function fail($error)
{
	http_response_code(500);
	include_once 'page.inc.php';
	ob_start();
	require_once '500.html';
	print getoutput(['debug' => $error]);
}

set_exception_handler('fail');

define('E_FATAL', E_ERROR | E_CORE_ERROR | E_COMPILE_ERROR | E_USER_ERROR);

set_error_handler(function ($level, $error, $file, $line) {
	if ($level & E_FATAL) {
		fail($error);
		return;
	}
	return FALSE;
});

register_shutdown_function(function () {
	# display failure page for fatal exceptions
	$error = error_get_last();
	if (!($error['type'] & E_FATAL)) return;
	fail("Fatal: $error[message] in $error[file]:$error[line]");
});

error_reporting(error_reporting() & ~E_FATAL);

# user login and control

$User = NULL;
include_once 'auth.inc.php';
$Edit = isset($_GET['edit']);

# distinguish subpage Args from topmost Page script

$Args = '';
$Page = preg_replace('/\?.*/', '', @$_SERVER['PATH_INFO'] ?: $_SERVER['REQUEST_URI']);
$Page = urldecode(trim($Page, '/')) ?: 'index';
while (TRUE) {
	if (file_exists("$Page/.private")) {
		# access restriction
		if (empty($User)) {
			http_response_code(303);
			$target = urlencode($_SERVER['REQUEST_URI']);
			header("Location: /login?goto=$target");
			exit;
		}
		$PageAccess = $Page;
	}

	if (file_exists("$Page/index.php")) {
		break;
	}

	$up = strrpos($Page, '/');
	$Args = substr($Page, $up) . $Args;
	$Page = substr($Page, 0, $up);
	if ($up === FALSE) {
		break;
	}
}

$staticpage = NULL;
if (file_exists("$Page$Args.html")) {
	$staticpage = "$Page$Args.html";
	if (is_link($staticpage)) {
		$target = preg_replace('/\.html$/', '', readlink($staticpage));
		header("HTTP/1.1 302 Shorthand");
		header("Location: $target");
		exit;
	}
}
elseif (file_exists("$Page$Args/index.html")) {
	$staticpage = "$Page$Args/index.html";
}
elseif ($User and $User->admin) {
	$staticpage = (file_exists("$Page/template.html") ? "$Page/template.html" : 'template.html');
}

# load static contents

require_once('article.inc.php');
$Article = new ArchiveArticle($staticpage);

ob_start(); # page body
ob_start(); # inner html
print '<div class="static">'."\n\n";

$found = FALSE;
if (isset($staticpage)) {
	$found = include "./$staticpage";
}

print "</div>\n\n";

# execute dynamic code

$Place = [];

if ($Page) {
	$found |= require "./$Page/index.php";
}

$Place += [
	'user'  => $User ? $User->login : '',
	'url'   => htmlspecialchars($_SERVER['REQUEST_URI']),
];

# global html

if (!$found) {
	# no resulting output
	http_response_code(404);
	@require '404.html';
}

include_once 'page.inc.php';

